@@ -29,8 +29,7 @@
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 #include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-about.h>
-#include <libgnomeui/gnome-window-icon.h>
+#include <gtk/gtkaboutdialog.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkimage.h>
@@ -53,11 +52,11 @@ typedef struct {
 
 	RbProxy *proxy;
 	gchar *song_tip;
+	gchar *time_tip;
 
 	GtkTooltips *tooltips;
 
 	GtkWidget *preferences_dialog;
-	GtkWidget *about_dialog;
 
 	GtkWidget *main_box;
 	GtkWidget *button_box;
@@ -86,6 +85,7 @@ static void set_initial_state (AppletData *data);
 
 static void display_rating (AppletData *data, int rating);
 static void enable_rating (AppletData *data, gboolean enable);
+static void recalculate_time_extents (AppletData *data);
 
 static void preferences_cb (BonoboUIComponent *uic,
 			    AppletData *data,
@@ -123,6 +123,9 @@ static void change_orient_cb (PanelApplet *applet,
 			      AppletData *data);
 static void destroy_cb (GtkWidget *widget,
 			AppletData *data);
+static void style_set_cb (GtkWidget *widget,
+			  GtkStyle *previous,
+			  AppletData *data);
 
 static void time_updated_cb (RbProxy *proxy,
 			     glong play_time,
@@ -154,8 +157,8 @@ rb_applet_fill (PanelApplet *applet,
 	if (strcmp (iid, "OAFIID:GNOME_Rhythmbox_Applet") != 0)
 		return FALSE;
 
-	gnome_window_icon_set_default_from_file (
-		PKG_DATA_DIR "rhythmbox.png");
+	gtk_window_set_default_icon_from_file (
+		PKG_DATA_DIR "rhythmbox.png", NULL);
 
 	data = g_new0 (AppletData, 1);
 	data->applet = GTK_WIDGET (applet);
@@ -166,9 +169,9 @@ rb_applet_fill (PanelApplet *applet,
 	data->proxy = rb_proxy_new ();
 	data->tooltips = gtk_tooltips_new ();
 	data->song_tip = NULL;
+	data->time_tip = NULL;
 	data->main_box = NULL;
 	data->preferences_dialog = NULL;
-	data->about_dialog = NULL;
 
 	/* Images */
 
@@ -266,6 +269,9 @@ rb_applet_fill (PanelApplet *applet,
 	g_signal_connect (applet, "destroy",
 			  G_CALLBACK (destroy_cb), data);
 
+	g_signal_connect (data->time, "style-set",
+			  G_CALLBACK (style_set_cb), data);
+
 	component = panel_applet_get_popup_component (PANEL_APPLET (applet));
 	g_signal_connect (component, "ui-event",
 			  G_CALLBACK (popup_component_event_cb), data);
@@ -273,6 +279,7 @@ rb_applet_fill (PanelApplet *applet,
 	/* Initialize with current state. */
 
 	set_initial_state (data);
+	recalculate_time_extents (data);
 
 	/* Finish up. */
 
@@ -287,7 +294,6 @@ set_initial_state (AppletData *data)
 	/* eew, hackish */
 
 	RbProxySongInfo *song = rb_proxy_get_song_info (data->proxy);
-	gboolean connected = rb_proxy_is_connected (data->proxy);
 	gboolean playing = rb_proxy_is_playing (data->proxy);
 	PanelAppletOrient orient = panel_applet_get_orient (
 		PANEL_APPLET (data->applet));
@@ -316,11 +322,22 @@ set_play_pause_stop_appearance (AppletData *data,
 static void
 display_rating (AppletData *data, int rating)
 {
+	/* NOTE: The rating widget and menu items *must* be enabled prior
+	 *       to this call, or else inconsistency could occur.
+	 */
+
 	BonoboUIComponent *component;
 	char path[] = "/commands/RateX";
+	int oldRating;
+	double oldRatingDouble;
 
 	/* Set the rating displayed by the rating widget. */
 
+	g_object_get (data->rating,
+		      "score", &oldRatingDouble,
+		      NULL);
+	oldRating = (int) oldRatingDouble;
+
 	g_object_set (data->rating,
 		      "score", (double) rating,
 		      NULL);
@@ -336,6 +353,23 @@ display_rating (AppletData *data, int rating)
 				      "state",
 				      "1",
 				      NULL);
+
+	/* You may ask, "Why do we need to explicitly set the state of the
+	 * previously selected item to 0?  These are radio menu items --
+	 * shouldn't that happen automatically?"  Yes, it should.  No, it
+	 * doesn't always do that (particularly if the menu has never been
+	 * displayed -- don't ask me why).
+	 */
+
+	if (oldRating != rating)
+	{
+		path[14] = (char) ('0' + oldRating);
+		bonobo_ui_component_set_prop (component,
+					      path,
+					      "state",
+					      "0",
+					      NULL);
+	}
 }
 
 static void
@@ -371,6 +405,22 @@ enable_rating (AppletData *data, gboolean enable)
 	}
 }
 
+static void 
+recalculate_time_extents (AppletData *data)
+{
+	PangoLayout *layout;
+	PangoRectangle extents;
+	int time_width;
+
+	/* should be as wide as it gets; assume digits are all same width */
+	layout = gtk_widget_create_pango_layout (data->time, "00:00");
+	pango_layout_get_pixel_extents (layout, NULL, &extents);
+	g_object_unref (layout);
+
+	gtk_widget_set_size_request (data->time, extents.width, -1);
+	gtk_container_resize_children (GTK_CONTAINER (data->main_box));
+}
+
 /* Callbacks for user actions. */
 
 static void
@@ -501,38 +551,15 @@ about_cb (BonoboUIComponent *uic,
 		NULL
 	};
 
-	static const gchar *documenters[] = {
-		NULL
-	};
-
-	const gchar *translator_credits = _("translator-credits");
-
-	if (data->about_dialog == NULL)
-	{
-		data->about_dialog = gnome_about_new (
-			_("Rhythmbox Applet"), PACKAGE_VERSION,
-			_("(C) 2004-2005 Paul Kuliniewicz"),
-			_("Control Rhythmbox from a GNOME panel."),
-			authors,
-			documenters,
-			strcmp (translator_credits, "translator-credits") != 0 ?
-				translator_credits : NULL,
-			NULL);
-
-		g_signal_connect (data->about_dialog,
-				  "destroy",
-				  G_CALLBACK (gtk_widget_destroyed),
-				  &data->about_dialog);
-
-		gtk_widget_show (data->about_dialog);
-	}
-	else
-	{
-		gtk_window_set_screen (GTK_WINDOW (data->about_dialog),
-				       gtk_widget_get_screen (
-					       GTK_WIDGET (data->applet)));
-		gtk_window_present (GTK_WINDOW (data->about_dialog));
-	}
+	gtk_show_about_dialog (NULL,
+		"name", _("Rhythmbox Applet"),
+		"version", PACKAGE_VERSION,
+		"copyright", _("(C) 2004-2005 Paul Kuliniewicz"),
+		"comments", _("Control Rhythmbox from a GNOME panel."),
+		"authors", authors,
+		"translator-credits", _("translator-credits"),
+		"website", "http://web.ics.purdue.edu/~kuliniew/rhythmbox-applet/",
+		NULL);
 }
 
 static void
@@ -646,6 +673,12 @@ destroy_cb (GtkWidget *widget, AppletData *data)
 	g_free (data);
 }
 
+static void
+style_set_cb (GtkWidget *widget, GtkStyle *previous_style, AppletData *data)
+{
+	recalculate_time_extents (data);
+}
+
 /* Callbacks for Rhythmbox events. */
 
 static void
@@ -653,15 +686,24 @@ time_updated_cb (RbProxy *proxy, glong play_time, AppletData *data)
 {
 	if (play_time >= 0)
 	{
-		glong minutes = play_time / 60;
-		glong seconds = play_time % 60;
-		gchar *text = g_strdup_printf ("%ld:%.2ld", minutes, seconds);
+		glong minutes;
+		glong seconds;
+		gchar *text;
+		gchar *time_text;
+
+		minutes = play_time / 60;
+		seconds = play_time % 60;
+
+		text = g_strdup_printf ("%ld:%.2ld", minutes, seconds);
 		gtk_label_set_text (GTK_LABEL (data->time), text);
 		g_free (text);
-		text = g_strdup_printf (data->song_tip, minutes, seconds);
+
+		time_text = g_strdup_printf (data->time_tip, minutes, seconds);
+		text = g_strdup_printf ("%s\n%s", data->song_tip, time_text);
 		gtk_tooltips_set_tip (data->tooltips, data->time_event,
 				      text, NULL);
 		g_free (text);
+		g_free (time_text);
 	}
 	else
 	{
@@ -675,49 +717,51 @@ static void
 song_changed_cb (RbProxy *proxy, const RbProxySongInfo *song, AppletData *data)
 {
 	g_free (data->song_tip);
+	g_free (data->time_tip);
 
 	if (song != NULL)
 	{
 		if (song->duration > 0)
 		{
 			/* playing local file */
+			gchar *whence;
 
-			gchar *whence = g_strdup_printf (
-				/* To translator: from ALBUM by ARTIST */
-				_("from %s by %s"),
-				song->album,
-				song->artist);
-
-			gchar *when = g_strdup_printf (
+			data->time_tip = g_strdup_printf (
 				/* To translator: CURRENT-TIME of TOTAL-TIME */
 				_("%%ld:%%.2ld of %d:%.2d"),
 				song->duration / 60,
 				song->duration % 60);
 
+			whence = g_strdup_printf (
+				/* To translator: from ALBUM by ARTIST */
+				_("from %s by %s"),
+				song->album,
+				song->artist);
+
 			data->song_tip = g_strdup_printf (
-				"%s\n%s\n%s",
+				"%s\n%s",
 				song->title,
-				whence,
-				when);
+				whence);
 
 			g_free (whence);
-			g_free (when);
 		}
 		else
 		{
 			/* playing Internet radio */
 
-			data->song_tip = g_strdup_printf (
-				"%s\n%%ld:%%.2ld",
-				song->title);
+			data->song_tip = g_strdup (song->title);
+			data->time_tip = g_strdup ("%ld:%.2ld");
 		}
 
+		enable_rating (data, TRUE);
 		display_rating (data, song->rating);
 	}
 	else
 	{
 		data->song_tip = g_strdup (_("Not playing"));
+		data->time_tip = g_strdup ("");
 		display_rating (data, 0);
+		enable_rating (data, FALSE);
 	}
 
 	time_updated_cb (proxy, rb_proxy_get_play_time (proxy), data);
@@ -748,7 +792,6 @@ playing_changed_cb (RbProxy *proxy, gboolean playing, AppletData *data)
 
 	has_song = playing || rb_proxy_is_pausable (proxy);
 
-	enable_rating (data, has_song);
 	gtk_widget_set_sensitive (data->previous, has_song);
 	gtk_widget_set_sensitive (data->next, has_song);
 	set_play_pause_stop_appearance (data, image, tip);
