@@ -494,6 +494,9 @@ playing_change_cb (BonoboListener *listener,
 					   NULL));
 
 	playing = BONOBO_ARG_GET_BOOLEAN (any);
+
+	if (rb_proxy_get_play_time (proxy) < 0)
+		g_signal_emit_by_name (proxy, "song_changed", NULL);
 	g_signal_emit_by_name (proxy, "playing_changed", playing);
 }
 
